/* ------------------main_srv.c ---------------------------
 * server using TCP protocol
 *
 * simple error handling
 */

#include "inet.h"
#include "sign.h"
#include "ipc_send.h"
#include "ipc_recv.h"

int main(int argc, char **argv) {

   int sockfd,newsockfd,clilen,childpid;
   struct sockaddr_in cli_addr, serv_addr;

   if( (ignoresig(SIGCHLD) == SIG_ERR) ) {
       perror("Signal");
       exit(1);
   }
   /*
    * open a TCP socket - Internet stream socket
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM,0)) < 0) {
      perror("server: can't open stream socket");
      exit(1);
   }

   /*
    * bind our local address so that the client can send us
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;

   serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
   /* INADR_ANY: tells the system that we'll accept a connection
    * on any Internet interface on the system, if it is multihomed
    * Address to accept any incoming messages (-> in.h).
    * INADDR_ANY is defined as ((unsigned long int) 0x00000000)
    */

   serv_addr.sin_port = htons(SERV_TCP_PORT);

   if ( bind(sockfd, (struct sockaddr *)&serv_addr, 
         sizeof(serv_addr)) < 0) {
      perror("server: can't bind local address");
      exit(1);
   }

   listen(sockfd,5);

   while(1) {
      /*
       * wait for a connection from a client process
       * - concurrent server -
       */
      clilen=sizeof(cli_addr);
      newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr, 
               &clilen); 
      
      if (newsockfd < 0 ) {
         if (errno == EINTR)
            continue; /*try again*/
         perror("server: accept error");
         exit(1);
      }

      if ( (childpid = fork()) < 0) {
         perror("server: fork error");
         close(sockfd);
         close(newsockfd);
         exit(1);
      } 
      else if (childpid == 0) {
         close(sockfd);
         /*****************************/
	 {
	    int a, b;
	    char op;
	    char buf[1024];

	    if (recvn(newsockfd, buf, sizeof(buf)) <= 0) {
	       perror("recvn");
	       exit(1);
	    }

	    if (sscanf(buf, "%c %d %d\n", &op, &a, &b) != 3)
	       sprintf(buf, "-Bad Request\n");
	    else if (op == '*')
	       sprintf(buf, "+%d\n", a * b);
	    else
	       sprintf(buf, "-Unknown Operator '%c'\n", op);

	    if (sendn(newsockfd, buf, strlen(buf)+1) < strlen(buf)+1) {
	       perror("ans");
	       exit(2);
	    }
	 }
         /*****************************/

         exit(0);
      }

      close(newsockfd); /*parent*/
   }
}
